"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const symbols_1 = require("./symbols");
const uuid_1 = require("uuid");
const Crypter = require("cryptr");
const libs_1 = require("./libs");
const services_1 = require("./services");
const DEBUG_KEY = 'airboat:common:decorators';
// tslint:disable-next-line: function-name
function Attribute(attribute = {}) {
    return (target, attributeKey) => {
        services_1.Logger.debug(JSON.stringify(target.constructor.name), DEBUG_KEY);
        const attributeProps = libs_1._.has(attribute, 'isJoi')
            ? { validation: attribute }
            : attribute;
        symbols_1.initClassDataSymbols(target);
        attributeProps.name = attributeProps.name || attributeKey;
        target[symbols_1.CLASS_DATA_ATTRIBUTES][attributeKey] = attributeProps;
    };
}
exports.Attribute = Attribute;
Attribute.model = (modelClass, options = { optional: false }, attribute = {}) => {
    const validation = options.optional
        ? libs_1.Joi.object().model().optional()
        : libs_1.Joi.object().model().required();
    return Attribute(libs_1._.defaults(attribute, {
        validation,
        deserialize: (o = {}) => new modelClass(o),
        serialize: (o) => o.serialize(),
    }));
};
Attribute.modelList = (model, options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.array().items(libs_1.Joi.object().model()).required(),
        deserialize: (list = []) => list.map((a) => new model(a)),
        serialize: (list) => list.map((a) => a.serialize()),
    }));
};
Attribute.dateTime = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.date().default(() => new Date(), 'now').optional(),
        deserialize: (o) => o ? new Date(o) : undefined,
        serialize: (o) => o.toISOString(),
    }));
};
Attribute.uuid = (options = { required: false }, attribute = {}) => {
    const validation = options.required
        ? libs_1.Joi.string().guid().required()
        : libs_1.Joi.string().guid().default(uuid_1.v4, 'new uuid').optional();
    return Attribute(libs_1._.defaults(attribute, { validation }));
};
Attribute.workspaceId = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().guid().required(),
    }));
};
Attribute.key = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(-[a-z]+)*$')).required(),
    }));
};
Attribute.keyDot = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(50).regex(new RegExp('^([a-z]*)(\.[a-z]+)*$')).required(),
    }));
};
Attribute.string = ({ max } = { max: 250 }, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(max).required(),
    }));
};
Attribute.oneOf = (validOptions, options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.any().valid(validOptions).required(),
    }));
};
Attribute.hash = (options = {}, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().max(500).required(),
    }));
};
Attribute.encryptedObject = ({ secret }, attribute = {}) => {
    return Attribute(libs_1._.defaults(attribute, {
        validation: libs_1.Joi.string().required(),
        deserialize: (v) => {
            const c = Crypter(secret);
            return JSON.parse(c.decrypt(v));
        },
        serialize: (v) => {
            const c = Crypter(secret);
            const s = c.encrypt(JSON.stringify(v));
        },
    }));
};
//# sourceMappingURL=decorators.js.map